/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "buffer.h"

static inline void libcjsonp_builtin_write8(buffer_t* x, unsigned char s) {
    *(x->ptr++) = s;
    --x->c;
}

static inline void libcjsonp_builtin_wrap8_8(buffer_t* x, unsigned char s) {
    *(x->ptr++) = '\\';
    *(x->ptr++) =    s;
    x->c -= 2;
}

static inline void libcjsonp_builtin_wrap8_e16(buffer_t* x, unsigned char s) {
    *(x->ptr++) = '\\';
    *(x->ptr++) =  'u';
    *(x->ptr++) =  '0';
    *(x->ptr++) =  '0';

    if ((s&0xf0) < 0xa0) {
        *(x->ptr++) = 0x30 + (s>>4);
    } else *(x->ptr++) = 0x57 + (s>>4);

    if ((s&0x0f) < 0x0a) {
        *(x->ptr++) = 0x30 + (s&0x0f);
    } else *(x->ptr++) = 0x57 + (s&0x0f);

    x->c -= 6;
}

void libcjsonp_builtin_buffer_write_and_wrap(buffer_t* x, const char* s) {
    unsigned char* v = (void*)s;

    buffer_check_size(x, 7);
    libcjsonp_builtin_write8(x, '"');

    while (*v) {
        if (*v < 0x20 || *v == 0x7f) switch (*v) {
            case 0x09: libcjsonp_builtin_wrap8_8  (x, 't'); break;
            case 0x0a: libcjsonp_builtin_wrap8_8  (x, 'n'); break;
            case 0x0d: libcjsonp_builtin_wrap8_8  (x, 'r'); break;
            default:   libcjsonp_builtin_wrap8_e16(x,  *v); break;
        } else if (*v == '\\') libcjsonp_builtin_wrap8_8(x, '\\');
          else if (*v ==  '"') libcjsonp_builtin_wrap8_8(x,  '"');
          else                 libcjsonp_builtin_write8 (x,   *v);

        ++v;

        buffer_check_size(x, 6);
    }

    buffer_write_char(x, '"');
}
