/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

extern const size_t *LIBCDSB_BUILTIN_VTYPE_SIZES;

indent_t LIBCJSONP_BUILTIN_WITHOUT_INDENT[1] = {{ .indent = 0, .nmemb = 0 }};

void libcjsonp_json_init(json_t* x, void* v, vtype t, bool attach) {
    value_t value;

    if (!attach) {
        switch (t) {
            default: break;
            case VTYPE_STRING: string_copy_init((void*)value.value, v); v = value.value; break;
            case    VTYPE_MAP:    map_copy_init((void*)value.value, v); v = value.value; break;
            case  VTYPE_ARRAY:  array_copy_init((void*)value.value, v); v = value.value; break;
            case   VTYPE_LIST:   list_copy_init((void*)value.value, v); v = value.value; break;
            case    VTYPE_SET:   vset_copy_init((void*)value.value, v); v = value.value; break;
            case   VTYPE_DICT:   dict_copy_init((void*)value.value, v); v = value.value; break;
        }
    }

    x->data = libcdsb_memndup(v, LIBCDSB_BUILTIN_VTYPE_SIZES[t]);
    x->type = t;
}


void json_free(json_t* x) {
    switch (x->type) {
        default: break;
        case VTYPE_STRING: string_free(x->data); break;
        case    VTYPE_MAP:    map_free(x->data); break;
        case  VTYPE_ARRAY:  array_free(x->data); break;
        case   VTYPE_LIST:   list_free(x->data); break;
        case    VTYPE_SET:   vset_free(x->data); break;
        case   VTYPE_DICT:   dict_free(x->data); break;
    }

    libcdsb_free(x->data);
    memset(x, 0, sizeof(*x));
}
